@@ -1,3 +1,4 @@
+//go:build particle_argon
 // +build particle_argon
 
 package machine
@@ -7,14 +8,6 @@ const HasLowFrequencyCrystal = true
 // More info: https://docs.particle.io/datasheets/wi-fi/argon-datasheet/
 // Board diagram: https://docs.particle.io/assets/images/argon/argon-block-diagram.png
 
-// LEDs
-const (
-	LED       Pin = 44
-	LED_GREEN Pin = 14
-	LED_RED   Pin = 13
-	LED_BLUE  Pin = 15
-)
-
 // GPIOs
 const (
 	A0  Pin = 3
@@ -39,10 +32,17 @@ const (
 	D13 Pin = 47 // Also SCK
 )
 
+// LEDs
+const (
+	LED       Pin = 44
+	LED_GREEN Pin = 14
+	LED_RED   Pin = 13
+	LED_BLUE  Pin = 15
+)
+
 // UART
 var (
-	Serial = &USB
-	UART0  = NRF_UART0
+	DefaultUART = UART0
 )
 
 const (
