@@ -1,14 +1,10 @@
+//go:build sam && atsamd21 && qtpy
 // +build sam,atsamd21,qtpy
 
 package machine
 
-import (
-	"device/sam"
-	"runtime/interrupt"
-)
-
 // used to reset into bootloader
-const RESET_MAGIC_VALUE = 0xf01669ef
+const resetMagicValue = 0xf01669ef
 
 // GPIO Pins
 const (
@@ -60,18 +56,7 @@ const (
 )
 
 // UART1 on the QT Py M0.
-var (
-	UART1  = &_UART1
-	_UART1 = UART{
-		Buffer: NewRingBuffer(),
-		Bus:    sam.SERCOM0_USART,
-		SERCOM: 0,
-	}
-)
-
-func init() {
-	UART1.Interrupt = interrupt.New(sam.IRQ_SERCOM0, _UART1.handleInterrupt)
-}
+var UART1 = &sercomUSART0
 
 // SPI pins
 const (
@@ -81,12 +66,7 @@ const (
 )
 
 // SPI on the QT Py M0.
-var (
-	SPI0 = SPI{
-		Bus:    sam.SERCOM0_SPI,
-		SERCOM: 0,
-	}
-)
+var SPI0 = sercomSPIM0
 
 // I2C pins
 const (
@@ -96,10 +76,7 @@ const (
 
 // I2C on the QT Py M0.
 var (
-	I2C0 = &I2C{
-		Bus:    sam.SERCOM1_I2CM,
-		SERCOM: 1,
-	}
+	I2C0 = sercomI2CM1
 )
 
 // I2S pins
@@ -119,3 +96,7 @@ var (
 	usb_VID uint16 = 0x239A
 	usb_PID uint16 = 0x80CB
 )
+
+var (
+	DefaultUART = UART1
+)
