@@ -1,14 +1,10 @@
+//go:build sam && atsamd21 && xiao
 // +build sam,atsamd21,xiao
 
 package machine
 
-import (
-	"device/sam"
-	"runtime/interrupt"
-)
-
 // used to reset into bootloader
-const RESET_MAGIC_VALUE = 0xf01669ef
+const resetMagicValue = 0xf01669ef
 
 // GPIO Pins
 const (
@@ -48,7 +44,7 @@ const (
 	LED3    = LED_TXL
 )
 
-// UART0 aka USBCDC pins
+// USBCDC pins
 const (
 	USBCDC_DM_PIN = PA24
 	USBCDC_DP_PIN = PA25
@@ -61,17 +57,7 @@ const (
 )
 
 // UART1 on the Xiao
-var (
-	UART1 = UART{
-		Buffer: NewRingBuffer(),
-		Bus:    sam.SERCOM4_USART,
-		SERCOM: 4,
-	}
-)
-
-func init() {
-	UART1.Interrupt = interrupt.New(sam.IRQ_SERCOM4, UART1.handleInterrupt)
-}
+var UART1 = &sercomUSART4
 
 // I2C pins
 const (
@@ -81,10 +67,7 @@ const (
 
 // I2C on the Xiao
 var (
-	I2C0 = &I2C{
-		Bus:    sam.SERCOM2_I2CM,
-		SERCOM: 2,
-	}
+	I2C0 = sercomI2CM2
 )
 
 // SPI pins
@@ -95,12 +78,7 @@ const (
 )
 
 // SPI on the Xiao
-var (
-	SPI0 = SPI{
-		Bus:    sam.SERCOM0_SPI,
-		SERCOM: 0,
-	}
-)
+var SPI0 = sercomSPIM0
 
 // I2S pins
 const (
@@ -109,11 +87,6 @@ const (
 	I2S_WS_PIN  = NoPin // TODO: figure out what this is on Xiao
 )
 
-// I2S on the Xiao
-var (
-	I2S0 = I2S{Bus: sam.I2S}
-)
-
 // USB CDC identifiers
 const (
 	usb_STRING_PRODUCT      = "Seeed XIAO M0"
