@@ -25,7 +25,9 @@ package runtime
 // heapStart..metadataStart.
 //
 // More information:
+// https://aykevl.nl/2020/09/gc-tinygo
 // https://github.com/micropython/micropython/wiki/Memory-Manager
+// https://github.com/micropython/micropython/blob/master/py/gc.c
 // "The Garbage Collection Handbook" by Richard Jones, Antony Hosking, Eliot
 // Moss.
 
@@ -35,11 +37,7 @@ import (
 	"unsafe"
 )
 
-// Set gcDebug to true to print debug information.
-const (
-	gcDebug   = false   // print debug info
-	gcAsserts = gcDebug // perform sanity checks
-)
+const gcDebug = false
 
 // Some globals + constants for the entire GC.
 
@@ -55,6 +53,9 @@ var (
 	metadataStart unsafe.Pointer // pointer to the start of the heap metadata
 	nextAlloc     gcBlock        // the next block that should be tried by the allocator
 	endBlock      gcBlock        // the block just past the end of the available space
+	gcTotalAlloc  uint64         // total number of bytes allocated
+	gcMallocs     uint64         // total number of allocations
+	gcFrees       uint64         // total number of objects freed
 )
 
 // zeroSizedAlloc is just a sentinel that gets returned when allocating 0 bytes.
@@ -259,12 +260,16 @@ func calculateHeapAddresses() {
 
 // alloc tries to find some free space on the heap, possibly doing a garbage
 // collection cycle if needed. If no space is free, it panics.
+//
 //go:noinline
 func alloc(size uintptr, layout unsafe.Pointer) unsafe.Pointer {
 	if size == 0 {
 		return unsafe.Pointer(&zeroSizedAlloc)
 	}
 
+	gcTotalAlloc += uint64(size)
+	gcMallocs++
+
 	neededBlocks := (size + (bytesPerBlock - 1)) / bytesPerBlock
 
 	// Continue looping until a run of free blocks has been found that fits the
@@ -281,7 +286,14 @@ func alloc(size uintptr, layout unsafe.Pointer) unsafe.Pointer {
 				// could be found. Run a garbage collection cycle to reclaim
 				// free memory and try again.
 				heapScanCount = 2
-				GC()
+				freeBytes := runGC()
+				heapSize := uintptr(metadataStart) - heapStart
+				if freeBytes < heapSize/3 {
+					// Ensure there is at least 33% headroom.
+					// This percentage was arbitrarily chosen, and may need to
+					// be tuned in the future.
+					growHeap()
+				}
 			} else {
 				// Even after garbage collection, no free memory could be found.
 				// Try to increase heap size.
@@ -372,6 +384,13 @@ func free(ptr unsafe.Pointer) {
 
 // GC performs a garbage collection cycle.
 func GC() {
+	runGC()
+}
+
+// runGC performs a garbage colleciton cycle. It is the internal implementation
+// of the runtime.GC() function. The difference is that it returns the number of
+// free bytes in the heap after the GC is finished.
+func runGC() (freeBytes uintptr) {
 	if gcDebug {
 		println("running collection cycle...")
 	}
@@ -413,12 +432,14 @@ func GC() {
 
 	// Sweep phase: free all non-marked objects and unmark marked objects for
 	// the next collection cycle.
-	sweep()
+	freeBytes = sweep()
 
 	// Show how much has been sweeped, for debugging.
 	if gcDebug {
 		dumpHeap()
 	}
+
+	return
 }
 
 // markRoots reads all pointers from start to end (exclusive) and if they look
@@ -561,7 +582,8 @@ func markRoot(addr, root uintptr) {
 }
 
 // Sweep goes through all memory and frees unmarked memory.
-func sweep() {
+// It returns how many bytes are free in the heap after the sweep.
+func sweep() (freeBytes uintptr) {
 	freeCurrentObject := false
 	for block := gcBlock(0); block < endBlock; block++ {
 		switch block.state() {
@@ -569,11 +591,14 @@ func sweep() {
 			// Unmarked head. Free it, including all tail blocks following it.
 			block.markFree()
 			freeCurrentObject = true
+			gcFrees++
+			freeBytes += bytesPerBlock
 		case blockStateTail:
 			if freeCurrentObject {
 				// This is a tail object following an unmarked head.
 				// Free it now.
 				block.markFree()
+				freeBytes += bytesPerBlock
 			}
 		case blockStateMark:
 			// This is a marked object. The next tail blocks must not be freed,
@@ -581,8 +606,11 @@ func sweep() {
 			// collect this object if it is unreferenced then.
 			block.unmark()
 			freeCurrentObject = false
+		case blockStateFree:
+			freeBytes += bytesPerBlock
 		}
 	}
+	return
 }
 
 // looksLikePointer returns whether this could be a pointer. Currently, it
