@@ -1,3 +1,4 @@
+//go:build stm32
 // +build stm32
 
 package runtime
@@ -6,13 +7,11 @@ import "device/arm"
 
 type timeUnit int64
 
-func postinit() {}
-
 //export Reset_Handler
 func main() {
 	preinit()
 	run()
-	abort()
+	exit(0)
 }
 
 func waitForEvents() {
