@@ -1,3 +1,4 @@
+//go:build stm32l0x1
 // +build stm32l0x1
 
 package runtime
@@ -14,7 +15,7 @@ const (
 func init() {
 	initCLK()
 
-	machine.Serial.Configure(machine.UARTConfig{})
+	machine.InitSerial()
 
 	initTickTimer(&machine.TIM21)
 }
